package cn.com.ty.lift.business.project.service;

import cn.com.ty.lift.business.project.dao.entity.Project;
import cn.com.ty.lift.business.project.dao.entity.ProjectHistory;
import cn.com.ty.lift.business.project.dao.entity.ProjectUser;
import cn.com.ty.lift.business.project.dao.entity.model.ProjectHistoryRequest;
import cn.com.ty.lift.business.project.dao.entity.model.ProjectIDResponse;
import cn.com.ty.lift.business.project.dao.entity.model.ProjectRequest;
import cn.com.ty.lift.business.project.dao.entity.model.ProjectResponse;
import cn.com.ty.lift.business.project.dao.mapper.ProjectHistoryMapper;
import cn.com.ty.lift.business.project.dao.mapper.ProjectMapper;
import cn.com.ty.lift.business.project.dao.mapper.ProjectUserMapper;
import cn.com.ty.lift.common.base.ExportRequest;
import cn.com.ty.lift.common.export.ExportUtils;
import cn.com.ty.lift.common.utils.DateUtils;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author bieao
 * @date 2019/11/28
 * @description 项目管理业务层
 */
@Service
public class ProjectService {

    @Resource
    private ProjectMapper projectMapper;

    @Resource
    private ProjectUserMapper projectUserMapper;

    @Resource
    private ProjectHistoryMapper projectHistoryMapper;


    private Map<String, String> paramMap = new HashMap<String, String>() {{
        put("projectCode", "项目编号");
        put("projectName", "项目名称");
        put("province", "省份");
        put("city", "市");
        put("district", "区");
        put("num", "台量");
        put("actualNum", "系统台量");
        put("address", "项目地址");
        put("startDate", "开始时间");
        put("endDate", "结束时间");
    }};

    /**
     * @param request 项目列表查询条件
     * @return RestResponse 结果集
     * @description 条件查询项目管理列表
     * @date 2019/11/27 2:10 PM
     */
    public IPage<ProjectResponse> list(ProjectRequest request) {
        IPage<ProjectResponse> page = new Page<>(request.getPageNum(), request.getPageSize());
        if (request.getCondition() != null) {
            request.setCondition(StrUtil.format("%{}%", request.getCondition()));
        }
        return projectMapper.findByCondition(page, request);
    }

    /**
     * @param request 公司项目查询条件
     * @return IPage<ProjectResponse> 公司项目分页列表结果
     * @description 查询公司项目列表
     * @date 2019/12/9 10:32 AM
     */
    public IPage<ProjectResponse> companyList(ProjectRequest request) {
        IPage<ProjectResponse> page = new Page<>(request.getPageNum(), request.getPageSize());
        return projectMapper.findCompanyListByCondition(page, request);
    }

    /**
     * @param request 项目操作记录查询条件
     * @return IPage<ProjectHistory> 结果集
     * @description 查询项目操作记录列表
     * @date 2019/11/27 2:10 PM
     */
    public IPage<ProjectHistory> list(ProjectHistoryRequest request) {
        IPage<ProjectHistory> page = new Page<>(request.getPageNum(), request.getPageSize());

        return projectHistoryMapper.findByCondition(page, request);

    }

    /**
     * @param request projectId 项目ID
     * @return List<ProjectUser> userList项目组成员列表
     * @description 查询项目组成员
     * @date 2019/12/13 2:40 PM
     */
    public List<ProjectUser> userList(ProjectRequest request) {
        return projectUserMapper.findUserListById(request.getId());
    }

    /**
     * @param request 区域id或维保公司id
     * @return List<ProjectIDResponse> 项目ID和名称列表
     * @description 项目ID和名称列表
     * @date 2019/12/16 4:21 PM
     */
    public List<ProjectIDResponse> projectIDList(ProjectRequest request) {
        return projectMapper.findProjectListByRegionId(request);
    }

    /**
     * @param id 项目id
     * @return Project 项目详情
     * @description 查询项目详情
     * @date 2019/12/6 10:51 AM
     */
    public Project detail(Long id) {
        return projectMapper.selectById(id);
    }

    /**
     * @param project 新增项目数据项
     * @return Integer 成功1或失败0
     * @description 新增项目
     * @date 2019/11/27 2:22 PM
     */
    public Integer add(Project project) {
        //按当前日期时间戳自动生成项目编号
        String projectCode = DateUtils.generateCode();
        project.setProjectCode("XM" + projectCode);
        return projectMapper.insert(project);
    }

    /**
     * @param projectUser 新增维保工数据项
     * @return Integer 成功1或失败0
     * @description 新增维保工
     * @date 2019/12/4 4:41 PM
     */
    public Integer add(ProjectUser projectUser) {
        return projectUserMapper.insert(projectUser);
    }

    /**
     * @param project 修改项目数据项
     * @return Integer 成功1或失败0
     * @description 修改项目
     * @date 2019/11/27 2:22 PM
     */
    public Integer modify(Project project) {
        return projectMapper.updateById(project);
    }

    /**
     * @param request 导出项目数据列表
     * @return RestResponse 状态码和返回消息
     * @description 导出项目
     * @date 2019/11/27 2:22 PM
     */
    public void export(ExportRequest request) {
        List<Project> projectList = projectMapper.selectBatchIds(request.getIds());
        ExportUtils utils = new ExportUtils();
        utils.export(projectList, paramMap, request.getExportPath());
    }
}
