package cn.com.ty.lift.enterprise.region.service.impl;

import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.enterprise.region.dao.entity.AreaCode;
import cn.com.ty.lift.enterprise.region.dao.entity.Region;
import cn.com.ty.lift.enterprise.region.dao.entity.model.RegionReq;
import cn.com.ty.lift.enterprise.region.dao.mapper.RegionMapper;
import cn.com.ty.lift.enterprise.region.service.RegionService;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 区域表 服务实现类
 * </p>
 *
 * @author wang-hai-cheng
 * @since 2019-12-09
 */
@Service
@AllArgsConstructor
public class RegionServiceImpl extends ServiceImpl<RegionMapper, Region> implements RegionService {
    private final AreaCodeServiceImpl areaCodeService;

    //分页区域列表
    public RestResponse regions(RegionReq req) {
        long pageNum = (req.getPageNum() - 1) * req.getPageSize();
        long pageSize = req.getPageSize();
        Page<Map<String, Object>> page = new Page<>(req.getPageNum(), pageSize);
        //获取维保团队的区域列表
        List<Map<String, Object>> regions = this.getBaseMapper().selectRegions(req.getMtCompanyId(), pageNum, pageSize);
        //如果没有区域
        if (regions.isEmpty()) {
            return RestResponse.ok(null, ApiConstants.RESULT_NO_DATA, "无数据");
        }
        List<Map<String, Object>> projects = this.getBaseMapper().selectProjects();
        List<Map<String, Object>> areaCodes = this.getBaseMapper().selectAreaCodes();
        int projectNum = 0;
        Integer num = 0;
        //遍历区域列表
        if (regions.isEmpty()) {
            return RestResponse.ok(null, ApiConstants.RESULT_NO_DATA, "无数据");
        }
        for (Map<String, Object> region : regions) {
            //遍历省市列表，将省市的id转化为name
            for (Map<String, Object> area : areaCodes) {
                if (region.get("provinceCode").equals(area.get("code"))) {
                    region.put("provinceName", area.get("name"));
                }
                if (region.get("cityCode").equals(area.get("code"))) {
                    region.put("cityName", area.get("name"));
                }
            }

            //遍历项目列表，获取区域下的项目个数和区域下的总电梯数
            Object id = region.get("id");
            for (Map<String, Object> project : projects) {
                if (project.get("regionId").equals(id)) {
                    projectNum += 1;
                    num += (Integer) project.get("num");
                }
            }
            region.put("projectNum", projectNum);
            region.put("num", num);
            projectNum = 0;
            num = 0;
        }
        //设置分页对象的属性
        int count = this.getBaseMapper().selectCount(new QueryWrapper<Region>()
                .eq("mt_company_id", req.getMtCompanyId()));
        page.setRecords(regions);
        page.setTotal(count);
        page.setPages(count / pageSize);
        return RestResponse.ok(page, ApiConstants.RESULT_SUCCESS, "查询成功");
    }

    //删除区域，如果区域下没有关联项目
    public RestResponse delete(Region region) {
        Map<String, Object> regionId = this.baseMapper.selectProjectCount(region.getId());
        if (regionId.get("count").equals(0L)) {
            if (this.removeById(region.getId())) {
                return RestResponse.ok(null, ApiConstants.RESULT_SUCCESS, "删除成功");
            }
        }
        return RestResponse.ok(null, ApiConstants.RESULT_ERROR, "区域内有项目，无法删除");
    }

    //项目模块用，查询区域列表和它属于的省市名称和code
    public RestResponse projectRegions(Region r) {
        List<Map<String, Object>> regions = this.baseMapper.selectProjectRegions(r.getId());
        if (regions.isEmpty()) {
            return RestResponse.ok(null, ApiConstants.RESULT_NO_DATA, "无数据");
        }
        List<AreaCode> areas = areaCodeService.list();
        for (Map<String, Object> region : regions) {
            for (AreaCode area : areas) {
                //如果区域的城市code等于区中的code，put名字到区域中
                if (region.get("cityCode").equals(area.getCode())) {
                    region.put("city", area.getName());
                }
                //如果区域的省code等于区中的code，put名字到区域中
                if (region.get("provinceCode").equals(area.getCode())) {
                    region.put("province", area.getName());
                }
            }
        }
        return RestResponse.ok(regions, ApiConstants.RESULT_SUCCESS, "查询成功");
    }

    //项目模块用，查询区域下的成员列表
    public RestResponse selectRegionUser(Region region) {
        region = this.getById(region.getId());
        if (region == null) {
            return RestResponse.ok(null, ApiConstants.RESULT_NO_DATA, "无此区域");
        }
        HashMap<String, Object> data = new HashMap<>();
        List<Map<String, Object>> regionUser = this.baseMapper.selectRegionMember(region.getUserId().toString());
        if (region.getClerk() != null && !region.getClerk().equals("-1")) {
            List<Map<String, Object>> regionClerk = this.baseMapper.selectRegionMember(region.getClerk());
            data.put("regionClerk", regionClerk);
        } else {
            data.put("regionClerk", null);
        }
        if (regionUser.get(0) == null) {
            data.put("regionUser", regionUser);
        } else {
            data.put("regionUser", regionUser.get(0).get("name"));
        }
        return RestResponse.ok(data, ApiConstants.RESULT_SUCCESS, "查询成功");
    }
}