package cn.com.ty.lift.quan.news.service.impl;

import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.common.utils.AliyunOSSUtil;
import cn.com.ty.lift.quan.news.dao.entity.QuanNews;
import cn.com.ty.lift.quan.news.dao.entity.model.QuanNewsReq;
import cn.com.ty.lift.quan.news.dao.mapper.QuanNewsMapper;
import cn.com.ty.lift.quan.news.service.QuanNewsService;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

/**
 * <p>
 * 电梯圈新闻公告 服务实现类
 * </p>
 *
 * @author wang-hai-cheng
 * @since 2019-12-02
 */
@Service
@AllArgsConstructor
public class QuanNewsServiceImpl extends ServiceImpl<QuanNewsMapper, QuanNews> implements QuanNewsService {

    private final RedisTemplate redisTemplate;
    private final StringRedisTemplate stringRedisTemplate;

    /**
     * @description 将新闻置顶
     * @date 2019/12/3 11:23
     */
    public RestResponse top(Long id, Integer type) {
        QuanNews byId = this.getById(id);
        if (byId == null) {
            return RestResponse.ok(null, "0", "新闻不存在");
        }
        //1置顶 0取消置顶
        if (type == 1) {
            if (byId.getTopSerial() != 0) {
                //如果新闻在置顶新闻中
                Integer serial = byId.getTopSerial();

                List<QuanNews> tops = this.list(new QueryWrapper<QuanNews>().ne("top_serial", 0));
                for (QuanNews top : tops) {
                    if (top.getTopSerial() < serial) {
                        top.setTopSerial(top.getTopSerial() + 1);
                    }
                }
                byId.setTopSerial(1);
                //更新排序
                this.updateBatchById(tops);
                this.updateById(byId);
            } else {
                //如果新闻不是置顶新闻
                //获取置顶新闻
                List<QuanNews> news = this.list(new QueryWrapper<QuanNews>().select("id", "top_serial").ne("top_serial", 0));
                if (!news.isEmpty()) {
                    //如果有置顶新闻
                    news.forEach(n -> {
                        n.setTopSerial(n.getTopSerial() + 1);
                    });
                    this.updateBatchById(news, 100);
                }
                byId.setTopSerial(1);
                //更新置顶新闻列表，新闻
                if (this.updateById(byId)) {
                    return RestResponse.ok(null, "1", "置顶成功");
                } else {
                    return RestResponse.ok(null, "0", "置顶失败");
                }
            }
            return RestResponse.ok(null, "1", "置顶成功");
        }
        //1置顶 0取消置顶
        if (type == 0) {
            if (byId.getTopSerial() == 0) {
                //如果不是置顶新闻
                return RestResponse.ok(null, "1", "不是置顶新闻");
            }
            List<QuanNews> tops = this.list(new QueryWrapper<QuanNews>().select("id", "top_serial").ne("top_serial", 0).ne("id", id));
            //如果置顶新闻为0个
            if (tops.isEmpty()) {
                byId.setTopSerial(0);
                if (this.updateById(byId)) {
                    return RestResponse.ok(null, "1", "取消置顶成功");
                } else {
                    return RestResponse.ok(null, "0", "取消置顶失败");
                }
            }
            //比这个新闻排序大的都 序号-1
            tops.forEach(top -> {
                if (top.getTopSerial() > byId.getTopSerial()) {
                    top.setTopSerial(top.getTopSerial() - 1);
                }
            });
            byId.setTopSerial(0);
            if (this.updateById(byId) && this.updateBatchById(tops)) {
                return RestResponse.ok(null, "1", "取消置顶成功");
            } else {
                return RestResponse.ok(null, "0", "取消置顶失败");
            }
        }
        return RestResponse.ok(null, "0", "请求参数有误");
    }

    /**
     * @description 查询置顶新闻+普通新闻列表
     * @date 2019/12/3 10:30
     */
    public RestResponse news(Long pageNum, Long pageSize, Integer haveTops) {
        Page<QuanNews> page = new Page<>(pageNum, pageSize);
        IPage<Map<String, Object>> news = this.pageMaps(page, new QueryWrapper<QuanNews>().eq("top_serial", "0").orderByDesc("release_date"));
        //全部新闻的浏览量列表
        Map<Object, Object> lookTimes = stringRedisTemplate.boundHashOps("system-quanNews-look-time").entries();
        //全部新闻的点赞用户列表
        Map<String, TreeSet> likeUsers = redisTemplate.opsForHash().entries("system-quanNews-like-users");
        if (haveTops == 0) {
            if (news.getRecords().isEmpty()) {
                return RestResponse.ok(null, ApiConstants.RESULT_NO_DATA, "无数据");
            }
            //把点赞的用户数量和浏览量整入新闻字段中
            newsAddLookAndLike(news.getRecords(), lookTimes, likeUsers);
            return RestResponse.ok(news, ApiConstants.RESULT_SUCCESS, "查询成功");
        }
        List<Map<String, Object>> topNews = this.listMaps(new QueryWrapper<QuanNews>().ne("top_serial", "0").orderByAsc("top_serial"));
        //把点赞的用户数量和浏览量整入新闻字段中
        newsAddLookAndLike(topNews, lookTimes, likeUsers);

        HashMap<String, Object> map = new HashMap<>();
        map.put("topNews", topNews);
        map.put("news", news);
        return RestResponse.ok(map, ApiConstants.RESULT_SUCCESS, "查询成功");
    }

    //把点赞的用户数量和浏览量整入新闻字段中
    void newsAddLookAndLike(List<Map<String, Object>> records, Map<Object, Object> lookTimes, Map<String, TreeSet> likeUsers) {
        for (Map<String, Object> record : records) {
            Object id = record.get("id");
            //如果没有浏览量
            if (lookTimes.isEmpty()) {
                record.put("lookNum", "0");
            } else {
                //遍历lookTimes，查看对应id的新闻浏览量
                for (Map.Entry<Object, Object> entry : lookTimes.entrySet()) {
                    Object key = entry.getKey();
                    Object value = entry.getValue();
                    if (key.equals(id.toString())) {
                        record.put("lookNum", value);
                        break;
                    }
                    record.put("lookNum", "0");
                }
            }
            //如果没有用户喜爱
            if (likeUsers.isEmpty()) {
                record.put("likeNum", "0");
            } else {
                //遍历likeUsers，查看对应id的新闻的点赞量
                for (Map.Entry<String, TreeSet> entry : likeUsers.entrySet()) {
                    String k = entry.getKey();
                    TreeSet v = entry.getValue();
                    if (k.equals(id.toString())) {
                        record.put("likeNum", v.size());
                        break;
                    }
                    record.put("likeNum", "0");
                }
            }
        }
    }

    public RestResponse addNew(QuanNewsReq req) {
        QuanNews quanNews = new QuanNews();
        quanNews.setTitle(req.getTitle())
                .setContent(req.getContent())
                .setReleaseId(req.getReleaseId())
                .setReleaseUser(req.getReleaseUser());

        List<QuanNews> tops;
        if (req.getIsTop() != 0) {
            //获取置顶新闻列表
            tops = this.list(new QueryWrapper<QuanNews>().ne("top_serial", 0));
            //设置新添加新闻的数据
            quanNews.setTopSerial(1);
            //遍历排序加1
            if (!tops.isEmpty()) {
                tops.forEach(top -> {
                    top.setTopSerial(top.getTopSerial() + 1);
                });
                if (this.updateBatchById(tops) && this.save(quanNews)) {
                    return RestResponse.ok(quanNews.getId(), "1", "新增成功");
                } else {
                    return RestResponse.ok(null, "0", "新增失败");
                }
            } else {
                if (this.save(quanNews)) {
                    return RestResponse.ok(quanNews.getId(), "1", "新增成功");
                } else {
                    return RestResponse.ok(null, "0", "新增失败");
                }
            }
        }
        //添加新闻
        if (this.save(quanNews)) {
            return RestResponse.ok(quanNews.getId(), "1", "新增成功");
        } else {
            return RestResponse.ok(null, "0", "新增失败");
        }
    }

    /**
     * @description 更新新闻
     * @date 2019/12/4 13:34
     */
    public RestResponse updateNews(QuanNews quanNews) {
        if (null == this.getById(quanNews.getId())) {
            return RestResponse.ok(null, "0", "不存在此新闻");
        }
        if (this.updateById(quanNews)) {
            return RestResponse.ok(null, "1", "更新成功");
        } else {
            return RestResponse.ok(null, "0", "更新失败");
        }
    }

    /**
     * @description 删除新闻
     * @date 2019/12/4 13:55
     */
    public RestResponse delete(Long id) {
        QuanNews byId = this.getById(id);
        if (null == byId) {
            return RestResponse.ok(null, "9", "无数据");

        }
        if (byId.getTopSerial() != 0) {
            //取消新闻置顶
            this.top(id, 0);
        }
        if (this.removeById(id)) {
            return RestResponse.ok(null, "1", "删除成功");
        } else {
            return RestResponse.ok(null, "0", "删除失败");
        }
    }

    //新增、更新新闻时上传文件
    public RestResponse<String> upFile(Long id, MultipartFile videoFile) {
        String originalFilename = videoFile.getOriginalFilename();
        if (originalFilename != null && originalFilename.contains(".")) {
            String substring = originalFilename.substring(videoFile.getOriginalFilename().lastIndexOf(".") + 1);
            String fileName = IdWorker.getIdStr() + "." + substring;

            boolean result = false;
            try {
                result = new AliyunOSSUtil().putObject("temp15827479607", videoFile.getBytes(), fileName);
            } catch (IOException e) {
                e.printStackTrace();
                return RestResponse.ok(null, ApiConstants.RESULT_ERROR, "失败");
            }
            if (result) {
                QuanNews quanNews = new QuanNews();
                quanNews.setId(id).setVideoName(fileName);
                this.updateById(quanNews);
                return RestResponse.ok(fileName, ApiConstants.RESULT_SUCCESS, "成功");
            }
            return RestResponse.ok(null, ApiConstants.RESULT_ERROR, "失败");
        }
        return RestResponse.ok(null, ApiConstants.RESULT_ERROR, "无数据或数据格式错误");
    }

    public RestResponse like(QuanNewsReq req) {
        String uid = req.getUserId().toString();
        String id = req.getId().toString();
        TreeSet users = (TreeSet) redisTemplate.opsForHash().get("system-quanNews-like-users", id);
        if (users == null) {
            users = new TreeSet<>();
            users.add(uid);
            redisTemplate.opsForHash().put("system-quanNews-like-users", id, users);
            return RestResponse.ok(1, ApiConstants.RESULT_SUCCESS, "点赞成功");
        } else {
            //遍历userId列表
            for (Object user : users) {
                if (user.equals(uid)) {
                    //如果列表内有这个uid就删除喜欢,并return
                    users.remove(user);
                    redisTemplate.opsForHash().put("system-quanNews-like-users", id, users);
                    return RestResponse.ok(0, ApiConstants.RESULT_SUCCESS, "取消点赞成功");
                }
            }
            //如果列表内没有这个uid就标记喜欢
            users.add(uid);
            redisTemplate.opsForHash().put("system-quanNews-like-users", id, users);
            return RestResponse.ok(1, ApiConstants.RESULT_SUCCESS, "点赞成功");
        }
    }

    //获取一条新闻
    public RestResponse getQuanNews(QuanNewsReq req) {
        Map<String, Object> byId = this.getMap(new QueryWrapper<QuanNews>().eq("id", req.getId()));
        if (null != byId) {
            String id = byId.get("id").toString();
            Long lookNum = redisTemplate.boundHashOps("system-quanNews-look-time").increment(id, 1);
            int isLike;
            TreeSet users = (TreeSet) redisTemplate.opsForHash().get("system-quanNews-like-users", id);
            if (users != null && !users.add(req.getUserId().toString())) {
                isLike = 1;
            } else {
                isLike = 0;
            }
            byId.put("lookNum", lookNum);
            byId.put("isLike", isLike);
            return RestResponse.ok(byId, ApiConstants.RESULT_SUCCESS, "查询成功");
        } else {
            return RestResponse.ok(null, ApiConstants.RESULT_NO_DATA, "无数据");
        }
    }
}